@@ -7,25 +7,22 @@ to authorize request which is being made to Firebase.
 import logging
 import typing as t
 import uuid
-from datetime import datetime, timedelta
+from datetime import datetime, timedelta, timezone
 from email.mime.nonmultipart import MIMENonMultipart
+from importlib.metadata import version
 from pathlib import PurePath
-from urllib.parse import urlencode, urljoin
+from urllib.parse import urlencode
 
 import httpx
 from google.oauth2 import service_account  # type: ignore
 
-import pkg_resources  # type: ignore
-from async_firebase._config import (
-    DEFAULT_REQUEST_LIMITS,
-    DEFAULT_REQUEST_TIMEOUT,
-    RequestLimits,
-    RequestTimeout,
-)
-from async_firebase.messages import FCMBatchResponse, FCMResponse
+from async_firebase._config import DEFAULT_REQUEST_LIMITS, DEFAULT_REQUEST_TIMEOUT, RequestLimits, RequestTimeout
+from async_firebase.messages import FCMBatchResponse, FCMResponse, TopicManagementResponse
 from async_firebase.utils import (
     FCMBatchResponseHandler,
     FCMResponseHandler,
+    TopicManagementResponseHandler,
+    join_url,
     serialize_mime_message,
 )
 
@@ -41,6 +38,10 @@ class AsyncClientBase:
     SCOPES: t.List[str] = [
         "https://www.googleapis.com/auth/cloud-platform",
     ]
+    IID_URL = "https://iid.googleapis.com"
+    IID_HEADERS = {"access_token_auth": "true"}
+    TOPIC_ADD_ACTION = "iid/v1:batchAdd"
+    TOPIC_REMOVE_ACTION = "iid/v1:batchRemove"
 
     def __init__(
         self,
@@ -49,6 +50,7 @@ class AsyncClientBase:
         *,
         request_timeout: RequestTimeout = DEFAULT_REQUEST_TIMEOUT,
         request_limits: RequestLimits = DEFAULT_REQUEST_LIMITS,
+        use_http2: bool = False,
     ) -> None:
         """
         :param credentials: instance of ``google.oauth2.service_account.Credentials``.
@@ -65,12 +67,25 @@ class AsyncClientBase:
         :param scopes: user-defined scopes to request during the authorization grant.
         :param request_timeout: advanced feature that allows to change request timeout.
         :param request_limits: advanced feature that allows to control the connection pool size.
+        :param use_http2: advanced feature that allows to control usage of http protocol.
         """
         self._credentials: service_account.Credentials = credentials
         self.scopes: t.List[str] = scopes or self.SCOPES
 
         self._request_timeout = request_timeout
         self._request_limits = request_limits
+        self._use_http2 = use_http2
+        self._http_client: t.Optional[httpx.AsyncClient] = None
+
+    @property
+    def _client(self) -> httpx.AsyncClient:
+        if self._http_client is None or self._http_client.is_closed:
+            self._http_client = httpx.AsyncClient(
+                timeout=httpx.Timeout(**self._request_timeout.__dict__),
+                limits=httpx.Limits(**self._request_limits.__dict__),
+                http2=self._use_http2,
+            )
+        return self._http_client
 
     def creds_from_service_account_info(self, service_account_info: t.Dict[str, str]) -> None:
         """
@@ -109,11 +124,10 @@ class AsyncClientBase:
             }
         ).encode("utf-8")
 
-        async with httpx.AsyncClient() as client:
-            response: httpx.Response = await client.post(self.TOKEN_URL, data=data, headers=headers)
-            response_data = response.json()
+        response: httpx.Response = await self._client.post(self.TOKEN_URL, content=data, headers=headers)
+        response_data = response.json()
 
-        self._credentials.expiry = datetime.utcnow() + timedelta(seconds=response_data["expires_in"])
+        self._credentials.expiry = datetime.now(timezone.utc) + timedelta(seconds=response_data["expires_in"])
         self._credentials.token = response_data["access_token"]
         return self._credentials.token
 
@@ -159,9 +173,44 @@ class AsyncClientBase:
             "Content-Type": "application/json; UTF-8",
             "X-Request-Id": self.get_request_id(),
             "X-GOOG-API-FORMAT-VERSION": "2",
-            "X-FIREBASE-CLIENT": "async-firebase/{0}".format(pkg_resources.get_distribution("async-firebase").version),
+            "X-FIREBASE-CLIENT": "async-firebase/{0}".format(version("async-firebase")),
         }
 
+    async def _send_request(
+        self,
+        url: str,
+        response_handler: t.Union[FCMResponseHandler, FCMBatchResponseHandler, TopicManagementResponseHandler],
+        json_payload: t.Optional[t.Dict[str, t.Any]] = None,
+        headers: t.Optional[t.Dict[str, str]] = None,
+        content: t.Union[str, bytes, t.Iterable[bytes], t.AsyncIterable[bytes], None] = None,
+    ) -> t.Union[FCMResponse, FCMBatchResponse, TopicManagementResponse]:
+        logging.debug(
+            "Requesting POST %s, payload: %s, content: %s, headers: %s",
+            url,
+            json_payload,
+            content,
+            headers,
+        )
+        try:
+            raw_fcm_response: httpx.Response = await self._client.post(
+                url,
+                json=json_payload,
+                headers=headers or await self.prepare_headers(),
+                content=content,
+            )
+            raw_fcm_response.raise_for_status()
+        except httpx.HTTPError as exc:
+            response = response_handler.handle_error(exc)
+        else:
+            logging.debug(
+                "Response Code: %s, Time spent to make a request: %s",
+                raw_fcm_response.status_code,
+                raw_fcm_response.elapsed,
+            )
+            response = response_handler.handle_response(raw_fcm_response)
+
+        return response
+
     async def send_request(
         self,
         uri: str,
@@ -171,7 +220,7 @@ class AsyncClientBase:
         content: t.Union[str, bytes, t.Iterable[bytes], t.AsyncIterable[bytes], None] = None,
     ) -> t.Union[FCMResponse, FCMBatchResponse]:
         """
-        Sends an HTTP call using the ``httpx`` library.
+        Sends an HTTP call using the ``httpx`` library to FCM.
 
         :param uri: URI to be requested.
         :param response_handler: the model to handle response.
@@ -180,34 +229,32 @@ class AsyncClientBase:
         :param content: request content
         :return: HTTP response
         """
-        async with httpx.AsyncClient(
-            base_url=self.BASE_URL,
-            timeout=httpx.Timeout(**self._request_timeout.__dict__),
-            limits=httpx.Limits(**self._request_limits.__dict__),
-        ) as client:
-            logging.debug(
-                "Requesting POST %s, payload: %s, content: %s, headers: %s",
-                urljoin(self.BASE_URL, self.FCM_ENDPOINT.format(project_id=self._credentials.project_id)),
-                json_payload,
-                content,
-                headers,
-            )
-            try:
-                raw_fcm_response: httpx.Response = await client.post(
-                    uri,
-                    json=json_payload,
-                    headers=headers or await self.prepare_headers(),
-                    content=content,
-                )
-                raw_fcm_response.raise_for_status()
-            except httpx.HTTPError as exc:
-                response = response_handler.handle_error(exc)
-            else:
-                logging.debug(
-                    "Response Code: %s, Time spent to make a request: %s",
-                    raw_fcm_response.status_code,
-                    raw_fcm_response.elapsed,
-                )
-                response = response_handler.handle_response(raw_fcm_response)
+        url = join_url(self.BASE_URL, uri)
+        return await self._send_request(  # type: ignore
+            url=url, response_handler=response_handler, json_payload=json_payload, headers=headers, content=content
+        )
 
-        return response
+    async def send_iid_request(
+        self,
+        uri: str,
+        response_handler: TopicManagementResponseHandler,
+        json_payload: t.Optional[t.Dict[str, t.Any]] = None,
+        headers: t.Optional[t.Dict[str, str]] = None,
+        content: t.Union[str, bytes, t.Iterable[bytes], t.AsyncIterable[bytes], None] = None,
+    ) -> TopicManagementResponse:
+        """
+        Sends an HTTP call using the ``httpx`` library to the IID service for topic management functionality.
+
+        :param uri: URI to be requested.
+        :param response_handler: the model to handle response.
+        :param json_payload: request JSON payload
+        :param headers: request headers.
+        :param content: request content
+        :return: HTTP response
+        """
+        url = join_url(self.IID_URL, uri)
+        headers = headers or await self.prepare_headers()
+        headers.update(self.IID_HEADERS)
+        return await self._send_request(  # type: ignore
+            url=url, response_handler=response_handler, json_payload=json_payload, headers=headers, content=content
+        )
