@@ -4,22 +4,21 @@ The module houses client to communicate with FCM - Firebase Cloud Messaging (And
 Documentation for google-auth package https://google-auth.readthedocs.io/en/latest/user-guide.html that is used
 to authorize request which is being made to Firebase.
 """
+import asyncio
+import collections
 import json
 import logging
 import typing as t
+import warnings
 from dataclasses import replace
-from datetime import datetime, timedelta
+from datetime import datetime, timedelta, timezone
 from email.mime.multipart import MIMEMultipart
 from email.mime.nonmultipart import MIMENonMultipart
 from urllib.parse import urljoin
 
 import httpx
 
-from async_firebase.base import (  # noqa: F401
-    AsyncClientBase,
-    RequestLimits,
-    RequestTimeout,
-)
+from async_firebase.base import AsyncClientBase, RequestLimits, RequestTimeout  # noqa: F401
 from async_firebase.encoders import aps_encoder
 from async_firebase.messages import (
     AndroidConfig,
@@ -33,6 +32,8 @@ from async_firebase.messages import (
     Message,
     MulticastMessage,
     PushNotification,
+    TopicManagementResponse,
+    Visibility,
     WebpushConfig,
     WebpushFCMOptions,
     WebpushNotification,
@@ -41,10 +42,12 @@ from async_firebase.messages import (
 from async_firebase.utils import (
     FCMBatchResponseHandler,
     FCMResponseHandler,
+    TopicManagementResponseHandler,
     cleanup_firebase_message,
     serialize_mime_message,
 )
 
+
 DEFAULT_TTL = 604800
 BATCH_MAX_MESSAGES = MULTICAST_MESSAGE_MAX_DEVICE_TOKENS = 500
 
@@ -74,7 +77,7 @@ class AsyncFirebaseClient(AsyncClientBase):
         has_apns_config = True if apns_config and apns_config.payload else False
         if has_apns_config:
             # avoid mutation of active message
-            message.apns = replace(message.apns)
+            message.apns = replace(message.apns)  # type: ignore
             message.apns.payload = aps_encoder(apns_config.payload.aps)  # type: ignore
 
         push_notification: t.Dict[str, t.Any] = cleanup_firebase_message(
@@ -106,6 +109,7 @@ class AsyncFirebaseClient(AsyncClientBase):
         title_loc_args: t.Optional[t.List[str]] = None,
         channel_id: t.Optional[str] = None,
         notification_count: t.Optional[int] = None,
+        visibility: Visibility = Visibility.PRIVATE,
     ) -> AndroidConfig:
         """
         Constructs AndroidConfig that will be used to customize the messages that are sent to Android device.
@@ -141,6 +145,7 @@ class AsyncFirebaseClient(AsyncClientBase):
         :param notification_count: The number of items in notification. May be displayed as a badge count for launchers
             that support badging. If zero or unspecified, systems that support badging use the default, which is to
             increment a number displayed on the long-press menu each time a new notification arrives (optional).
+        :param visibility: set the visibility of the notification. The default level, VISIBILITY_PRIVATE.
         :return: an instance of ``messages.AndroidConfig`` to be included in the resulting payload.
         """
         android_config = AndroidConfig(
@@ -163,6 +168,7 @@ class AsyncFirebaseClient(AsyncClientBase):
                 title_loc_args=title_loc_args or [],
                 channel_id=channel_id,
                 notification_count=notification_count,
+                visibility=visibility,
             ),
         )
 
@@ -227,7 +233,7 @@ class AsyncFirebaseClient(AsyncClientBase):
         """
 
         apns_headers = {
-            "apns-expiration": str(int(datetime.utcnow().timestamp()) + ttl),
+            "apns-expiration": str(int(datetime.now(timezone.utc).timestamp()) + ttl),
             "apns-priority": str(10 if priority == "high" else 5),
         }
         if apns_topic:
@@ -415,6 +421,9 @@ class AsyncFirebaseClient(AsyncClientBase):
 
         :return: instance of ``messages.FCMBatchResponse``
         """
+        warnings.warn(
+            "send_multicast is going to be deprecated, please use send_each_for_multicast instead", DeprecationWarning
+        )
 
         if len(multicast_message.tokens) > MULTICAST_MESSAGE_MAX_DEVICE_TOKENS:
             raise ValueError(
@@ -430,6 +439,7 @@ class AsyncFirebaseClient(AsyncClientBase):
                 android=multicast_message.android,
                 webpush=multicast_message.webpush,
                 apns=multicast_message.apns,
+                fcm_options=multicast_message.fcm_options,
             )
             for token in multicast_message.tokens
         ]
@@ -450,6 +460,7 @@ class AsyncFirebaseClient(AsyncClientBase):
             without actually delivering the message. Default to ``False``.
         :returns: instance of ``messages.FCMBatchResponse``
         """
+        warnings.warn("send_all is going to be deprecated, please use send_each instead", DeprecationWarning)
 
         if len(messages) > BATCH_MAX_MESSAGES:
             raise ValueError(f"A list of messages must not contain more than {BATCH_MAX_MESSAGES} elements")
@@ -492,3 +503,95 @@ class AsyncFirebaseClient(AsyncClientBase):
         if not isinstance(batch_response, FCMBatchResponse):
             raise ValueError("Wrong return type, perhaps because of a response handler misuse.")
         return batch_response
+
+    async def send_each(
+        self,
+        messages: t.Union[t.List[Message], t.Tuple[Message]],
+        *,
+        dry_run: bool = False,
+    ) -> FCMBatchResponse:
+        if len(messages) > BATCH_MAX_MESSAGES:
+            raise ValueError(f"Can not send more than {BATCH_MAX_MESSAGES} messages in a single batch")
+
+        push_notifications = [
+            self.assemble_push_notification(apns_config=message.apns, dry_run=dry_run, message=message)
+            for message in messages
+        ]
+
+        request_tasks: t.Collection[collections.abc.Awaitable] = [
+            self.send_request(
+                uri=self.FCM_ENDPOINT.format(project_id=self._credentials.project_id),  # type: ignore
+                json_payload=push_notification,
+                response_handler=FCMResponseHandler(),
+            )
+            for push_notification in push_notifications
+        ]
+        fcm_responses = await asyncio.gather(*request_tasks)
+        return FCMBatchResponse(responses=fcm_responses)
+
+    async def send_each_for_multicast(
+        self,
+        multicast_message: MulticastMessage,
+        *,
+        dry_run: bool = False,
+    ) -> FCMBatchResponse:
+        if len(multicast_message.tokens) > MULTICAST_MESSAGE_MAX_DEVICE_TOKENS:
+            raise ValueError(
+                f"A single ``messages.MulticastMessage`` may contain up to {MULTICAST_MESSAGE_MAX_DEVICE_TOKENS} "
+                "device tokens."
+            )
+
+        messages = [
+            Message(
+                token=token,
+                data=multicast_message.data,
+                notification=multicast_message.notification,
+                android=multicast_message.android,
+                webpush=multicast_message.webpush,
+                apns=multicast_message.apns,
+                fcm_options=multicast_message.fcm_options,
+            )
+            for token in multicast_message.tokens
+        ]
+
+        return await self.send_each(messages, dry_run=dry_run)
+
+    async def _make_topic_management_request(
+        self, device_tokens: t.List[str], topic_name: str, action: str
+    ) -> TopicManagementResponse:
+        payload = {
+            "to": f"/topics/{topic_name}",
+            "registration_tokens": device_tokens,
+        }
+        response = await self.send_iid_request(
+            uri=action,
+            json_payload=payload,
+            response_handler=TopicManagementResponseHandler(),
+        )
+        return response
+
+    async def subscribe_devices_to_topic(self, device_tokens: t.List[str], topic_name: str) -> TopicManagementResponse:
+        """
+        Subscribes devices to the topic.
+
+        :param device_tokens: devices ids to be subscribed.
+        :param topic_name: name of the topic.
+        :returns: Instance of messages.TopicManagementResponse.
+        """
+        return await self._make_topic_management_request(
+            device_tokens=device_tokens, topic_name=topic_name, action=self.TOPIC_ADD_ACTION
+        )
+
+    async def unsubscribe_devices_from_topic(
+        self, device_tokens: t.List[str], topic_name: str
+    ) -> TopicManagementResponse:
+        """
+        Unsubscribes devices from the topic.
+
+        :param device_tokens: devices ids to be unsubscribed.
+        :param topic_name: name of the topic.
+        :returns: Instance of messages.TopicManagementResponse.
+        """
+        return await self._make_topic_management_request(
+            device_tokens=device_tokens, topic_name=topic_name, action=self.TOPIC_REMOVE_ACTION
+        )
