@@ -4,10 +4,25 @@
 
 import typing as t
 from dataclasses import dataclass, field
+from enum import IntEnum
+
+import httpx
 
 from async_firebase.errors import AsyncFirebaseError
 
 
+class Visibility(IntEnum):
+    """Available visibility levels.
+
+    To get more insights please follow the reference
+    https://developer.android.com/reference/android/app/Notification#visibility
+    """
+
+    PRIVATE: int = 0
+    PUBLIC: int = 1
+    SECRET: int = -1
+
+
 @dataclass
 class AndroidNotification:
     """
@@ -37,6 +52,8 @@ class AndroidNotification:
     notification_count: the number of items this notification represents (optional). If zero or unspecified, systems
         that support badging use the default, which is to increment a number displayed on the long-press menu each time
         a new notification arrives.
+    visibility: sets the different visibility levels of a notification. More about Visibility levels can be found by
+        reference https://firebase.google.com/docs/reference/fcm/rest/v1/projects.messages#Visibility
     """
 
     title: t.Optional[str] = None
@@ -52,6 +69,7 @@ class AndroidNotification:
     title_loc_args: t.List[str] = field(default_factory=list)
     channel_id: t.Optional[str] = None
     notification_count: t.Optional[int] = None
+    visibility: t.Optional[Visibility] = None
 
 
 @dataclass
@@ -280,6 +298,17 @@ class WebpushConfig:
     fcm_options: t.Optional[WebpushFCMOptions] = field(default=None)
 
 
+@dataclass
+class FcmOptions:
+    """
+    Platform independent options for features provided by the FCM SDKs
+    Arguments:
+        analytics_label: Label associated with the message's analytics data.
+    """
+
+    analytics_label: str
+
+
 @dataclass
 class Message:
     """
@@ -297,6 +326,7 @@ class Message:
     token: the registration token of the device to which the message should be sent.
     topic: name of the Firebase topic to which the message should be sent (optional).
     condition: the Firebase condition to which the message should be sent (optional).
+    fcm_options: platform independent options for features provided by the FCM SDKs.
     """
 
     token: t.Optional[str] = None
@@ -307,6 +337,7 @@ class Message:
     apns: t.Optional[APNSConfig] = field(default=None)
     topic: t.Optional[str] = None
     condition: t.Optional[str] = None
+    fcm_options: t.Optional[FcmOptions] = field(default=None)
 
 
 @dataclass
@@ -321,6 +352,7 @@ class MulticastMessage:
     android: an instance of ``messages.AndroidConfig`` (optional).
     webpush: an instance of ``messages.WebpushConfig`` (optional).
     apns: an instance of ``messages.ApnsConfig`` (optional).
+    fcm_options: platform independent options for features provided by the FCM SDKs.
     """
 
     tokens: t.List[str]
@@ -329,6 +361,7 @@ class MulticastMessage:
     android: t.Optional[AndroidConfig] = field(default=None)
     webpush: t.Optional[WebpushConfig] = field(default=None)
     apns: t.Optional[APNSConfig] = field(default=None)
+    fcm_options: t.Optional[FcmOptions] = field(default=None)
 
 
 @dataclass
@@ -395,3 +428,62 @@ class FCMBatchResponse:
     @property
     def failure_count(self):
         return len(self.responses) - self.success_count
+
+
+class TopicManagementErrorInfo:
+    """An error encountered when performing a topic management operation."""
+
+    def __init__(self, index, reason):
+        self._index = index
+        self._reason = reason
+
+    @property
+    def index(self):
+        """Index of the registration token to which this error is related to."""
+        return self._index
+
+    @property
+    def reason(self):
+        """String describing the nature of the error."""
+        return self._reason
+
+
+class TopicManagementResponse:
+    """The response received from a topic management operation."""
+
+    def __init__(self, resp: t.Optional[httpx.Response] = None, exception: t.Optional[AsyncFirebaseError] = None):
+        self.exception = exception
+        self._success_count = 0
+        self._failure_count = 0
+        self._errors: t.List[TopicManagementErrorInfo] = []
+
+        if resp:
+            self._handle_response(resp)
+
+    def _handle_response(self, resp: httpx.Response):
+        response = resp.json()
+        results = response.get("results")
+        if not results:
+            raise ValueError("Unexpected topic management response: {0}.".format(resp))
+
+        for index, result in enumerate(results):
+            if "error" in result:
+                self._failure_count += 1
+                self._errors.append(TopicManagementErrorInfo(index, result["error"]))
+            else:
+                self._success_count += 1
+
+    @property
+    def success_count(self):
+        """Number of tokens that were successfully subscribed or unsubscribed."""
+        return self._success_count
+
+    @property
+    def failure_count(self):
+        """Number of tokens that could not be subscribed or unsubscribed due to errors."""
+        return self._failure_count
+
+    @property
+    def errors(self):
+        """A list of ``messaging.ErrorInfo`` objects (possibly empty)."""
+        return self._errors
