@@ -8,6 +8,8 @@ from email.generator import Generator
 from email.mime.multipart import MIMEMultipart
 from email.mime.nonmultipart import MIMENonMultipart
 from email.parser import FeedParser
+from enum import Enum
+from urllib.parse import quote, urlencode, urljoin
 
 import httpx
 
@@ -23,7 +25,42 @@ from async_firebase.errors import (
     UnknownError,
     UnregisteredError,
 )
-from async_firebase.messages import FCMBatchResponse, FCMResponse
+from async_firebase.messages import FCMBatchResponse, FCMResponse, TopicManagementResponse
+
+
+def join_url(
+    base: str,
+    *parts: t.Union[str, int],
+    params: t.Optional[dict] = None,
+    leading_slash: bool = False,
+    trailing_slash: bool = False,
+) -> str:
+    """Construct a full ("absolute") URL by combining a "base URL" (base) with another URL (url) parts.
+
+    :param base: base URL part
+    :param parts: another url parts that should be joined
+    :param params: dict with query params
+    :param leading_slash: flag to force leading slash
+    :param trailing_slash: flag to force trailing slash
+
+    :return: full URL
+    """
+    url = base
+    if parts:
+        quoted_and_stripped_parts = [quote(str(part).strip("/"), safe=": /") for part in parts]
+        url = "/".join([base.strip("/"), *quoted_and_stripped_parts])
+
+    # trailing slash can be important
+    if trailing_slash:
+        url = f"{url}/"
+    # as well as a leading slash
+    if leading_slash:
+        url = f"/{url}"
+
+    if params:
+        url = urljoin(url, "?{}".format(urlencode(params)))
+
+    return url
 
 
 def remove_null_values(dict_value: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
@@ -87,6 +124,8 @@ def cleanup_firebase_message(dataclass_obj, dict_factory: t.Callable = dict) ->
         result = []
         for f in fields(dataclass_obj):
             value = cleanup_firebase_message(getattr(dataclass_obj, f.name), dict_factory)
+            if isinstance(value, Enum):
+                value = value.value
             result.append((f.name, value))
         return remove_null_values(dict_factory(result))
     elif isinstance(dataclass_obj, (list, tuple)):
@@ -122,11 +161,10 @@ def serialize_mime_message(
     return fp.getvalue()
 
 
-FCMResponseType = t.TypeVar("FCMResponseType", FCMResponse, FCMBatchResponse)
+FCMResponseType = t.TypeVar("FCMResponseType", FCMResponse, FCMBatchResponse, TopicManagementResponse)
 
 
 class FCMResponseHandlerBase(ABC, t.Generic[FCMResponseType]):
-
     ERROR_CODE_TO_EXCEPTION_TYPE: t.Dict[str, t.Type[AsyncFirebaseError]] = {
         FcmErrorCode.INVALID_ARGUMENT.value: errors.InvalidArgumentError,
         FcmErrorCode.FAILED_PRECONDITION.value: errors.FailedPreconditionError,
@@ -320,3 +358,20 @@ class FCMBatchResponseHandler(FCMResponseHandlerBase[FCMBatchResponse]):
             responses.append(resp)
 
         return responses
+
+
+class TopicManagementResponseHandler(FCMResponseHandlerBase[TopicManagementResponse]):
+    def handle_error(self, error: httpx.HTTPError) -> TopicManagementResponse:
+        exc = (
+            (isinstance(error, httpx.HTTPStatusError) and self._handle_fcm_error(error))
+            or (isinstance(error, httpx.HTTPError) and self._handle_request_error(error))
+            or AsyncFirebaseError(
+                code=FcmErrorCode.UNKNOWN.value,
+                message="Unexpected error has happened when hitting the FCM API",
+                cause=error,
+            )
+        )
+        return TopicManagementResponse(exception=exc)
+
+    def handle_response(self, response: httpx.Response) -> TopicManagementResponse:
+        return TopicManagementResponse(response)
